﻿using System;

namespace Samurai.Workflow
{
    public class DelegatedWorkflowStep<T> : WorkflowStepBase<T> where T : class, IWorkflowContext 
    {
        private readonly Action<T> _action;
        private readonly Action _parameterlessAction;

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DelegatedWorkflowStep&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="action">The action.</param>
        public DelegatedWorkflowStep(Action<T> action)
        {
            if (action == null) throw new ArgumentNullException("action");
            _action = action;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DelegatedWorkflowStep&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="action">The action.</param>
        public DelegatedWorkflowStep(Action action)
        {
            if (action == null) throw new ArgumentNullException("action");
            _parameterlessAction = action;
        }

        #endregion

        #region Overrides of WorkflowStepBase<T>

        /// <summary>
        /// Executes the step.
        /// </summary>
        public override void ExecuteStep()
        {
            try
            {
                if(_parameterlessAction != null)
                    _parameterlessAction.Invoke();
                else
                    _action.Invoke(Context);

                NotifyStepCompleted();
            }
            catch (Exception e)
            {
                NotifyStepFailed(e);
            }
        }

        #endregion
    }
}
