﻿using System;
using System.Collections.Generic;

namespace Samurai.Workflow
{
    public class WorkflowCompletedEventArgs<T> : EventArgs where T : class, IWorkflowContext 
    {
        /// <summary>
        /// Gets or sets the context.
        /// </summary>
        /// <value>The context.</value>
        public T Context { get; set; }

        /// <summary>
        /// Gets or sets the exception that occured during workflow execution.
        /// </summary>
        /// <value>The exception.</value>
        public Exception Exception { get; set; }

        /// <summary>
        /// Gets or sets any additional information about the workflow execution. For example, if 
        /// the workflow failed for multiple reasons, they can be listed here.
        /// </summary>
        /// <value>The additional information.</value>
        public IList<string> AdditionalInformation { get; set; }

        /// <summary>
        /// Gets or sets the failed workflow step.
        /// </summary>
        /// <value>The failed workflow step.</value>
        public IWorkflowStep<T> FailedWorkflowStep { get; set; }
    }
}
